@@ -10,6 +10,7 @@ from collections import defaultdict
 import ast
 import cache
 import compare
+import contains
 import compact
 import util
 from ast import dup
@@ -145,6 +146,11 @@ def refactor(pred_set, ast, settings=None):
     if settings.refactor:
         ast = recursive_refactor(ast, settings)
 
+    # Perform static resolution of all literals again, since
+    # literal sets may have changed
+    if settings.static_rewrite:
+        static_resolution(ast, pred_set)
+
     # Compact the tree
     if settings.compact:
         compact.compact(ast)
@@ -161,7 +167,7 @@ def static_resolution(ast, pred):
     def resolve_func(pattern, literal):
         literal.static_resolve(pred)
 
-    pattern = SimplePattern("types:Literal")
+    pattern = SimplePattern("types:Literal,LiteralSet")
     tile(ast, [pattern], resolve_func)
 
 
@@ -222,6 +228,11 @@ def select_rewrite_expression(name, exprs):
     if name[0] == "CompareOperator":
         return compare.select_rewrite_expression(name, exprs)
 
+    # Check if this is a contains operator. The contains operator
+    # uses static rewriting if we are operating on LiteralSets
+    elif name[0] == "ContainsOperator" and name[1] == 'LiteralSet':
+        return contains.select_rewrite_expression(name, exprs)
+
     # For negate operators, use the sub-expression
     elif isinstance(exprs[0], ast.NegateOperator):
         return exprs[0].left
@@ -245,6 +256,9 @@ def rewrite_ast(node, name, expr, assumed_result):
     if name[0] == "CompareOperator":
         return compare.compare_rewrite(node, name, expr, assumed_result)
 
+    elif name[0] == "ContainsOperator" and name[1] == 'LiteralSet':
+        return contains.contains_rewrite(node, name, expr, assumed_result)
+
     else:
         # Tile over the AST and replace the expresssion
         const = ast.Constant(assumed_result)
@@ -289,6 +303,9 @@ def node_name(node, enable_static=False):
         else:
             return (cls_name, node.value)
 
+    elif cls_name == "LiteralSet":
+        return cls_name
+
     elif cls_name == "Number":
         if enable_static:
             return (cls_name, "static")
@@ -341,7 +358,7 @@ def count_patterns():
     if CACHE_PATTERNS:
         return CACHE_PATTERNS
 
-    simple_types = "types:Literal,Number,Constant,Undefined,Empty"
+    simple_types = "types:Literal,LiteralSet,Number,Constant,Undefined,Empty"
 
     # Handle a negation of a simple type
     p1 = SimplePattern("types:NegateOperator", "types:Literal")
